/* grdzip.h.  Generated from grdzip.h.in by configure.  */
/*	$Id: grdzip.h.in,v 1.20 2012/10/18 00:25:07 myself Exp $
 *
 * grdzip.h contains declarations for grdzip.c.
 *
 * This software is described in more detail in the article
 *
 * "Compression of Large Data Grids for Internet Transmission", by Paul Wessel,
 *    Computers & Geosciences, 29 (5), 665-671, 2003.
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; version 2 of the License.
 */
 
#ifndef _GRDZIP_H
#define _GRDZIP_H

#define _POSIX_SOURCE 1

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>

#define PROGRAM_VERSION "2.11"
#if defined(WIN32) || defined(__EMX__)	/* Some definitions and includes are different under Windows or OS/2 */
#pragma warning( disable : 4996 )
#define WORDS_BIGENDIAN 0	/* Hardwired for PCs */
#define STAT _stat
#include <io.h>
#include <direct.h>
#define chdir(path) _chdir(path)
#define getcwd(path, len) _getcwd(path, len)
#else					/* Here for Unix, Linux, Cygwin, Interix, etc */
#include <unistd.h>
#define STAT stat
#define WORDS_BIGENDIAN 0	/* Determined by configure instead */
#endif
#ifndef __EMX__
#include <fcntl.h>
#endif

#ifdef _WIN64
typedef unsigned __int64 ULONG;	/* An unsigned 8-byte integer */
#else
typedef unsigned long ULONG;	/* An unsigned 4 (or 8-byte for 64-bit) integer */
#endif

#include "bzlib.h"		/* The bz2lib include file */

/* Macros for byte-order swapping (From John M. Kuhn, NOAA) */
 
#define swab_2(data) ((((data) & 0xff) << 8) | ((unsigned short) (data) >> 8))
#define swab_4(data) \
        (((data) << 24) | (((data) << 8) & 0x00ff0000) | \
        (((data) >> 8) & 0x0000ff00) | ((unsigned int)(data) >> 24))
	
#define GRDZIP_VERSION	8	/* < 128. Cannot contain lowest bit (i.e. 1) */
#define HEADER_SIZE	892	/* Size of the GMT grdheader struct, unfortunately not multiple of 8 */

#ifndef TRUE
#define TRUE	1
#endif
#ifndef FALSE
#define FALSE	0
#endif

/*-----------------------------------
 * THIS IS TAKEN AS IS FROM GMT_GRD.H
 *----------------------------------*/

#define GRD_COMMAND_LEN	320
#define GRD_REMARK_LEN	160
#define GRD_TITLE_LEN	 80
#define GRD_UNIT_LEN	 80

struct GRD_HEADER {
	int nx;				/* Number of columns */
	int ny;				/* Number of rows */
	int node_offset;		/* 0 for node grids, 1 for pixel grids */
	double x_min;			/* Minimum x coordinate */
	double x_max;			/* Maximum x coordinate */
	double y_min;			/* Minimum y coordinate */
	double y_max;			/* Maximum y coordinate */
	double z_min;			/* Minimum z value */
	double z_max;			/* Maximum z value */
	double x_inc;			/* x increment */
	double y_inc;			/* y increment */
	double z_scale_factor;		/* grd values must be multiplied by this */
	double z_add_offset;		/* After scaling, add this */
	char x_units[GRD_UNIT_LEN];	/* units in x-direction */
	char y_units[GRD_UNIT_LEN];	/* units in y-direction */
	char z_units[GRD_UNIT_LEN];	/* grid value units */
	char title[GRD_TITLE_LEN];	/* name of data set */
	char command[GRD_COMMAND_LEN];	/* name of generating command */
	char remark[GRD_REMARK_LEN];	/* comments re this data set */
};

typedef char BOOLEAN;              /* BOOLEAN used for logical variables */

#define N_FORMATS 		17

struct GEOZIP {
	unsigned int id;	/* Identifier for the format */
	unsigned int bits;	/* Number of left shifts for additional increments increment */
	unsigned int size;	/* Maximum size of additional increments */
	unsigned int sign_bit;	/* Bit to turn on if subsequent increments sign is -1 */
	unsigned int n_in_row;	/* Number of increments stored in 32-bits using this format */
	unsigned int format_id;	/* Format flag (the highest 4 bits in the 32-bit word */
	unsigned int n_bytes;	/* Number of bytes used for this encoding string */
};

/* Here are the 16 layouts or formats used to pack increments (+ spillover 17th format for large values) */

struct GEOZIP template[N_FORMATS] = {
	{  0,  0,     0,           0,     19,  0 <<  4, 1},	/* Format for strings of 5-19 zeros BYTE encoded */
	{  1,  0,     0,           0,      4,  1 <<  4, 1},	/* Format for strings of 4 0 or +1 BYTE encoded */
	{  2,  2, (1 <<  1) - 1, 1 <<  1,  6,  2 << 12, 2},	/* Format for  6  2-bit values |z| <= 1 2-BYTE encoded */ 
	{  3,  3, (1 <<  2) - 1, 1 <<  2,  4,  3 << 12, 2},	/* Format for  4  3-bit values |z| <= 3 2-BYTE encoded */ 
	{  4,  2, (1 <<  1) - 1, 1 <<  1, 10,  4 << 20, 3},	/* Format for 10  2-bit values |z| <=      1 3-BYTE encoded */
	{  5,  4, (1 <<  3) - 1, 1 <<  3,  5,  5 << 20, 3},	/* Format for  5  4-bit values |z| <=      7 3-BYTE encoded */
	{  6,  5, (1 <<  4) - 1, 1 <<  4,  4,  6 << 20, 3},	/* Format for  4  5-bit values |z| <=     15 3-BYTE encoded */
	{  7,  6, (1 <<  5) - 1, 1 <<  5,  3,  7 << 20, 3},	/* Format for  3  6-bit values |z| <=     31 3-BYTE encoded */
	{  8, 10, (1 <<  9) - 1, 1 <<  9,  2,  8 << 20, 3},	/* Format for  2 10-bit values |z| <=    511 3-BYTE encoded */
	{  9, 20, (1 << 19) - 1, 1 << 19,  1,  9 << 20, 3},	/* Format for  1 20-bit values |z| <= 524287 3-BYTE encoded */
	{ 10,  2, (1 <<  1) - 1, 1 <<  1, 14, 10 << 28, 4},	/* Format for 14  2-bit values |z| <= 1 4-BYTE encoded */
	{ 11,  3, (1 <<  2) - 1, 1 <<  2,  9, 11 << 28, 4},	/* Format for  9  3-bit values |z| <= 3 4-BYTE encoded */
	{ 12,  4, (1 <<  3) - 1, 1 <<  3,  7, 12 << 28, 4},	/* Format for  7  4-bit values |z| <= 7 4-BYTE encoded */
	{ 13,  7, (1 <<  6) - 1, 1 <<  6,  4, 13 << 28, 4},	/* Format for  4  7-bit values |z| <= 63 4-BYTE encoded */
	{ 14,  9, (1 <<  8) - 1, 1 <<  8,  3, 14 << 28, 4},	/* Format for  3  9-bit values |z| <= 255 4-BYTE encoded */
	{ 15, 14, (1 << 13) - 1, 1 << 13,  2, 15 << 28, 4},	/* Format for  2 14-bit values |z| <= 8191 4-BYTE encoded */
	{ 16, 32,    32,           0,      1,   127,    5},	/* Format for  1 32-bit values |z| > 524287 5-BYTE encoded */
};
#endif
