.TH GRDZIP l "1 Apr 2012"
.SH NAME
grdzip \- Pack/unpack 1-, 2-, or 4-byte integer data grids
.SH SYNOPSIS
\fBgrdzip\fP [ \fIinfile\fP] [ \fB\-C|D\fP ] [ \fB\-\fP1|2|4 ] [ \fB\-E\fP\fIendian\fP ] [ \fB\-F\fP\fIoutfile\fP ] [ \fB\-H\fP ] 
[ \fB\-I\fP\fIin_size\fP ] [ \fB\-N\fP ] [ \fB\-Q\fP ] [ \fB\-S\fP ] [ \fB\-V\fP ] [ \fB\-X\fP\fInx\fP ] [ \fB\-Y\fP\fInx\fP ]
.SH DESCRIPTION
\fBgrdzip\fP reads a binary 1-, 2-, or 4-byte signed integer grid (or GMT byte, short int or int grid; see \fB\-H\fP)
(or stdin) and encodes it using
the d2zdxdy differences of the data.  These increments are stored in a packed 8/16/24/32-bit format that varies
with the size of the increments.  \fBgrdzip\fP takes advantage of the fact that many data sets are
smooth, so the differences from point to point are small and can be stored in much less than 16 or 32 bits.
The endianness of the calling computer is recorded at the start of the file so that grdzip can
make any required byte swapping during unpacking.  The packed data is written to file (or stdout) via the bzip2
i/o library, resulting in superior data compression.
For technical details, see grdzip.c.
.SH OPTIONS FOR COMPRESSION
.TP
\fIinfile\fP
binary, integer file in scanline format [we read stdin if no file is given]. 
.TP
.B \-1
Selects 1-byte integer grid (i.e., char).
.TP
.B \-2
Selects 2-byte integer grid (i.e., short int) [Default].
.TP
.B \-4
Selects 4-byte integer grid (i.e., int).
.TP
.B \-C
Selects compression [Default].
.TP
.B \-F
Specifies output file [Default is stdout].
.TP
.B \-H
Input file has a GMT header structure [Default assumes no header].
.TP
.B \-S
Performs a byte-swap on incoming data.  This is needed when the endian-ness of the
data is different from that of the computer (Ignored if -1 is selected).
.TP
.B \-V
Report progress and final statistics to stderr [Default is silent].
.TP
.B \-X
Number of data columns, i.e., points along each scanline (not needed if header is present).
.TP
.B \-Y
Number of data rows, i.e., scanlines  (not needed if header is present).
.SH EXAMPLES OF COMPRESSION
To compress the global topography file etopo5.i2, try
.br
.sp
grdzip etopo5.i2 \fB\-V \-X\fP4321 \fB\-X\fP2160 -Fetopo5.gbz
.br
.sp
To pack Sandwell and Smith's 2 min gravity img file really small, try
.br
.sp
grdzip grav7.2.img \fB\-V \-X\fP10800 \fB\-X\fP6336 > grav7.2.img.gbz
.br
.sp
The combined packing and compression reduces this 131 Mb file to approximately 25% of its size.

.SH OPTIONS FOR DECOMPRESSION
.TP
\fIinfile\fP
Packed file made by \fBgrdzip \-C\fP [we read stdin if no file is given]. 
.TP
.B \-D
Selects decompression [Default is compression].
.TP
.B \-E
Select desired endian-ness of the output file.  Choose \fIB\fP for big-endian,
\fIL\fP for little-endian, \fIO\fP for the the original endian-ness on the originating
machine, or \fIH\fP for the host-system's endian-ness [Default].  Ignored if file
is a byte grid.
.TP
.B \-F
Specifies output file [Default is stdout].
.TP
.B \-I
If stdin is read we need the packed file's actual input
size in order to report on expansion/compression ratio.
.TP
.B \-N
Do NOT write a GMT header structure if it is present in the compressed file [Default will write such headers].
.TP
.B \-Q
Do not extract data, just report what kind (byte, short, or int) of grid, its size, and whether or not it
has a GMT header.
.TP
.B \-V
Report progress and final statistics to stderr [Default is silent].
.SH EXAMPLES OF DECOMPRESSION
To unpack the packed Sandwell and Smith's 2 min gravity img used in the
example above, and make sure it is written in big-endian format, try
.br
.sp
grdzip grav7.2.img.gbz \fB\-D \-EB -V\fP > grav7.2.img
.SH REFERENCE
Wessel, P., 2003, Compression of large data grids for Internet transmission,
Computers & Geosciences, 29 (5), 665-671.
